"""Import functions
"""
import argparse
import logging
import os
import re
from typing import Dict, List

_logger = logging.getLogger(__name__)


def add_importing_file_parsing(parser: argparse.ArgumentParser):
    # TODO indicate that either a file or a directory is required
    parser.add_argument("-f", "--file", help="CSV to import", action="append")
    parser.add_argument(
        "--directory", help="Directory of CSV files to import", action="append"
    )


def extract_model_lang_from_parsed(
    namespace: argparse.Namespace, expected_filenames: str = None
) -> List[Dict[str, str]]:
    pattern = re.compile(
        r"(?P<filename>"
        r"(?P<path>.*/)?"
        r"(?:[\d\s\w_-]*\s)?"
        r"(?P<model>[\w.]+?)"
        r"(?:@(?P<lang>\w{2,3}(?:@latin|_\w{2})))?"
        r".csv"
        r")"
    )
    expected_filenames_pattern = (
        re.compile(expected_filenames)
        if expected_filenames is not None
        else None
    )
    return extract_info_from_parsed(
        namespace, pattern, expected_filenames_pattern
    )


def extract_model_table_from_parsed(
    namespace: argparse.Namespace,
) -> List[Dict[str, str]]:
    pattern = re.compile(
        r"(?P<filename>"
        r"(?P<path>.*/)?"
        r"(?:[\d\s\w_-]*\s)?"
        r"(?P<model>[\w.]+?)"
        r"(?:@(?P<table>\w+))?"
        r".csv"
        r")"
    )
    expected_filenames_pattern = None
    return extract_info_from_parsed(
        namespace, pattern, expected_filenames_pattern
    )


def extract_info_from_parsed(
    nmspc: argparse.Namespace, pattern, expected_filenames_pattern=None
) -> List[Dict[str, str]]:
    """
    :param nmspc: result of parser.parse_args
    :param expected_filenames: a pattern to use in re.compile
    :return: list of (model, filename, lang)
    """
    return extract_info_from_filename(
        nmspc.file, nmspc.directory, pattern, expected_filenames_pattern
    )


def extract_info_from_filename(
    files, directories, pattern, expected_filenames_pattern=None
) -> List[Dict[str, str]]:
    """from filename or directories, extract model and lang

    :param files:
    :param directories:
    :param pattern: a re.compile pattern to match the file agains
    :param expected_filenames_pattern: a re.compile pattern
    :return: list of dict with the group data in it
    """
    model_filenames: List[Dict[str, str]] = list()
    file_list = list()
    if files:
        file_list.extend(files)
    if directories:
        for directory in directories:
            for root, subdirectories, files in os.walk(
                directory, followlinks=True
            ):
                file_list.extend(os.path.join(root, f) for f in sorted(files))

    for filename in file_list:
        _logger.debug("%s to import", filename)
        result = pattern.fullmatch(filename)
        if result:
            model_filenames.append(result.groupdict())
        else:
            if (
                expected_filenames_pattern
                and expected_filenames_pattern.match(filename)
            ):
                _logger.debug(
                    'Expected file name "%s" found, ignored', filename
                )
            else:
                _logger.warning(
                    'Unrecognized file name "%s", ignored', filename
                )
    return model_filenames
